// Class definitions for the Magzdiarz and Zdziarski Compton reflection models

#include <xsTypes.h>
#include <XSstreams.h>
#include <functionMap.h>
#include <FunctionUtility.h>
#include <XSUtil/FunctionUtils/xsFortran.h>
#include <XSUtil/Utils/XSutility.h>
#include <cfortran.h>

using namespace std;

//  ranges for different methods
#define XMIN 2.E-4                    // = 0.1022 keV
#define XTRANSL 0.01957               // = 10.00027 keV
#define XTRANSH 0.02348               // = 11.99828 keV
#define YMIN 0.03333                  // = 15,330 keV

#define XJL  log(XTRANSL)
#define XJD (log(XTRANSH) - XJL)

#define SQ3 1.732051

// class definition

class MZCompRefl{
 public:

  MZCompRefl();         // default constructor
  ~MZCompRefl();        // default destructor

  void CalcReflection(string RootName, Real cosIncl, Real xnor, Real Xmax,
		      RealArray& InputX, RealArray& InputSpec, RealArray& Spref);

 private:

  RealArray X;               // Input energies (m_e c^2 units)
  RealArray Spinc;           // Input spectrum

  RealArray pm1;
  RealArray pmx;
  RealArray ap;

  Real pmymax;

  Real Xmax;
  Real Precision;
  Real y;

  void InterpolatedSpectrum(RealArray& y, RealArray& sout);  // return array of spectra for given 1/X values
  void InterpolatedValue(Real y, Real& sout);  // return single spectrum value for input 1/X.

  void WriteSpectrum(string filename);  // for diagnostic purposes - write X, Spinc arrays to file

  void IntegrandSetup(Real y);
  void CalculateIntegrand(RealArray& y0, RealArray& OutArray);

  void GaussKronrodIntegral(Real LowerLimit, Real UpperLimit, Real& Integral,
			    Real& IntegralError);
  int AdaptiveIntegral(Real LowerLimit, Real UpperLimit, Real& Integral,
			Real& IntegralError);

};

// C++ functions used by these classes

void calcCompReflTotalFlux(string ModelName, Real Scale, Real cosIncl, Real Abund, 
			   Real FeAbund, Real Xi, Real Temp, Real inXmax, 
			   RealArray& X, RealArray& Spinc, RealArray& Sptot);
void CalcNonRelComp(Real cosIncl, RealArray& X, RealArray& opac, RealArray& NonRel);
void grxy(RealArray& pm1, RealArray& pmx, Real y, RealArray& y0, RealArray& ap, RealArray& grout);
Real aic(Real y0, Real dy, Real apc);
void pmxy(Real Xm, RealArray& Pmx);
void pm1y(Real Xm, RealArray& Pm1);
Real apf (Real x);

